import { Logger, TableCellValue, WorkflowType, IRunnableClient, components, WorkflowResponse, WorkflowId } from '@runnablejs/api';
export { IRunnableClient } from '@runnablejs/api';

interface RunnableAppContext {
    /**
     * Websocket
     * @example ws://localhost:8080
     */
    wsUrl: string;
    logger?: Logger;
}
interface RunnableContext {
    logger: Logger;
    user: User;
}
interface User {
    id: string;
    name?: string;
    email: string;
}

type ValidationResponse = string | true;
type Validator<T> = (value: T) => ValidationResponse;

type InputForm<T extends object> = {
    [P in keyof T]: FormPromise<T[P]>;
};
interface FormFieldOptions<T> {
    label: string;
    helperText?: string;
    placeholder?: string;
    optional?: boolean;
    defaultValue?: T;
    validation?: Validator<T>;
}
interface FormPromise<T> extends Promise<T> {
    payload: any;
    cancel: () => void;
}
interface MultiSelectOptions<T> extends FormFieldOptions<T[]> {
    initialSelection?: string[];
    data: T[];
    getLabel: (item: T) => string;
    getValue: (item: T) => string;
}
interface SelectOptions<T> extends FormFieldOptions<T> {
    initialSelection?: string;
    data: T[];
    getLabel: (item: T) => string;
    getValue: (item: T) => string;
}
interface TextOptions extends FormFieldOptions<string> {
    type?: 'text' | 'password' | 'email';
}
interface NumberOptions extends FormFieldOptions<number> {
}
interface BooleanOptions extends FormFieldOptions<boolean> {
}
interface ColorOptions extends FormFieldOptions<string> {
}
interface ImageOptions extends FormFieldOptions<string> {
}
interface TableOptions<T> extends FormFieldOptions<T> {
    data: T[];
    headers: string[];
    initialSelection?: string;
    getValue: (item: T) => string;
    getColumns: (item: T) => TableCellValue[];
}
interface MultiTableOptions<T> extends FormFieldOptions<T[]> {
    data: T[];
    headers: string[];
    initialSelection?: string[];
    getValue: (item: T) => string;
    getColumns: (item: T) => TableCellValue[];
}
interface CombinedForm {
    <A, B>(a: FormPromise<A>, b: FormPromise<B>): FormPromise<[A, B]>;
    <A, B, C>(a: FormPromise<A>, b: FormPromise<B>, c: FormPromise<C>): FormPromise<[A, B, C]>;
    <A, B, C, D>(a: FormPromise<A>, b: FormPromise<B>, c: FormPromise<C>, d: FormPromise<D>): FormPromise<[A, B, C, D]>;
}
/**
 * Input/Output for an action
 *
 * Promises are sent to the client and resolved when the user submits a response.
 *
 * @example
 * ```
 * const { email } = await io.input.text({
 *  label: 'Email',
 *  description: 'The email address of the user',
 * });
 */
interface InputOutput {
    form<T extends Record<string, any>>(form: InputForm<T>): FormPromise<T>;
    hstack: CombinedForm;
    vstack: CombinedForm;
    input: {
        text(opts: TextOptions): FormPromise<string>;
        number(opts: NumberOptions): FormPromise<number>;
        boolean(opts: BooleanOptions): FormPromise<boolean>;
        color(opts: ColorOptions): FormPromise<string>;
        imageURL(opts: ImageOptions): FormPromise<string>;
    };
    select: {
        radio<T>(opts: SelectOptions<T>): FormPromise<T>;
        dropdown<T>(opts: SelectOptions<T>): FormPromise<T>;
        table<T>(opts: TableOptions<T>): FormPromise<T>;
    };
    multiSelect: {
        checkbox<T>(opts: MultiSelectOptions<T>): FormPromise<T[]>;
        dropdown<T>(opts: MultiSelectOptions<T>): FormPromise<T[]>;
        table<T>(opts: MultiTableOptions<T>): FormPromise<T[]>;
    };
    message: {
        html(opts: {
            dangerouslySetInnerHTML: string;
        }): void;
        info(opts: {
            title: string;
            message: string;
        }): FormPromise<void>;
        table(opts: {
            title: string;
            headers: string[];
            rows: TableCellValue[][];
        }): FormPromise<void>;
        success(opts: {
            title: string;
            message: string;
        }): FormPromise<void>;
        warning(opts: {
            title: string;
            message: string;
        }): FormPromise<void>;
        error(opts: {
            title: string;
            message: string;
        }): FormPromise<void>;
    };
}

/**
 * Mapped workflows
 */
interface RunnableWorkflows {
    [key: string]: RunnableWorkflow;
}
/**
 * Single runnable workflow
 */
interface RunnableWorkflow extends Omit<WorkflowType, 'id'> {
    /**
     * Action name
     * @example 'Edit User'
     */
    title: string;
    /**
     * Action description
     * @example 'Edit a user's permissions, name, and email.'
     */
    description: string;
    /**
     * Icon to display in the UI
     * These can be Iconify icons, e.g. fa6-solid:user
     * You can find more icons here: https://iconify.design/icon-sets/
     */
    icon: string;
    /**
     * Category to display in the UI
     * @example 'Users'
     * @default 'Other'
     */
    category: string;
    /**
     * Execute the action
     */
    execute: (io: InputOutput, context: RunnableContext) => Promise<void>;
}

declare class Runnable implements IRunnableClient {
    private workflows;
    private workflowManager;
    constructor(workflows: RunnableWorkflows);
    listWorkflowTypes(): Promise<{
        workflows: components['schemas']['WorkflowType'][];
    }>;
    startWorkflow(workflowTypeId: string): Promise<WorkflowResponse>;
    pickUpWorkflow(workflowId: WorkflowId): Promise<WorkflowResponse>;
    continueWorkflow(workflowId: WorkflowId, response: {
        [key: string]: unknown;
    }): Promise<WorkflowResponse>;
}

interface StartOptions {
    /**
     * Host to connect to
     * e.g `ws://localhost:3000`
     */
    runnableHost: string;
    /**
     * Namespace. This is used to dedupe between multiple instances of the same server
     * e.g. 'user-server'
     */
    namespace: string;
    /**
     * Optional logger to use for logging
     * @default console
     */
    logger?: Logger;
    /**
     * Optional auth token to check against x-runnable-auth header
     * This is useful if you want to run the server on a public network
     *
     * @default '0123456789'
     */
    token?: string;
}
/**
 * The Runnable websocket client that connects to a Runnable server
 */
declare class RunnableWs {
    private workflows;
    private socket;
    private logger;
    private delegate;
    constructor(workflows: RunnableWorkflows);
    start(options: StartOptions): void;
    destroy(): void;
}

interface ClientResponse<T = any> {
    ioResponse: T;
}

interface WorkflowManager {
    /**
     * Start a workflow.
     * @param action The action to start the workflow with.
     * @param context The context to start the workflow with.
     * @returns The response to the action.
     */
    startWorkflow(action: RunnableWorkflow, context: RunnableContext): Promise<WorkflowResponse>;
    /**
     * Pick up a workflow.
     * @param workflowId The ID of the workflow to pick up.
     * @returns The response to the action.
     * @throws If the workflow doesn't exist.
     */
    pickUpWorkflow(workflowId: WorkflowId): Promise<WorkflowResponse>;
    /**
     * Continue a workflow.
     * @param workflowId The ID of the workflow to continue.
     * @param request The request to continue the workflow with.
     */
    continueWorkflow(workflowId: WorkflowId, request: ClientResponse): Promise<WorkflowResponse>;
}

declare class InMemoryWorkflowManager implements WorkflowManager {
    private mapWorkflows;
    private getWorkflow;
    startWorkflow(workflowType: RunnableWorkflow, context: RunnableContext): Promise<WorkflowResponse>;
    pickUpWorkflow(workflowId: WorkflowId): Promise<WorkflowResponse>;
    continueWorkflow(workflowId: WorkflowId, request: ClientResponse): Promise<WorkflowResponse>;
}

export { InMemoryWorkflowManager, InputOutput, Runnable, RunnableAppContext, RunnableWorkflow, RunnableWorkflows, RunnableWs };
